@@ -37,6 +37,7 @@ if TYPE_CHECKING:
 # [START hitl_tutorial]
 
 
+# [START hitl_notifier]
 class LocalLogNotifier(BaseNotifier):
     """Simple notifier to demonstrate HITL notification without setup any connection."""
 
@@ -46,10 +47,14 @@ class LocalLogNotifier(BaseNotifier):
         self.message = message
 
     def notify(self, context: Context) -> None:
+        url = HITLOperator.generate_link_to_ui_from_context(
+            context=context,
+            base_url="http://localhost:28080",
+        )
         self.log.info(self.message)
+        self.log.info("Url to respond %s", url)
 
 
-# [START htil_notifer]
 hitl_request_callback = LocalLogNotifier(
     message="""
 [HITL]
@@ -65,7 +70,7 @@ hitl_success_callback = LocalLogNotifier(
     message="{% set task_id = task.task_id -%}{{ ti.xcom_pull(task_ids=task_id) }}"
 )
 hitl_failure_callback = LocalLogNotifier(message="Request to response to '{{ task.subject }}' failed")
-# [END htil_notifer]
+# [END hitl_notifier]
 
 with DAG(
     dag_id="example_hitl_operator",
@@ -95,7 +100,7 @@ with DAG(
     )
     # [END howto_hitl_operator]
 
-    # [START howto_hitl_operator_muliple]
+    # [START howto_hitl_operator_multiple]
     wait_for_multiple_options = HITLOperator(
         task_id="wait_for_multiple_options",
         subject="Please choose option to proceed: ",
@@ -105,7 +110,7 @@ with DAG(
         on_success_callback=hitl_success_callback,
         on_failure_callback=hitl_failure_callback,
     )
-    # [END howto_hitl_operator_muliple]
+    # [END howto_hitl_operator_multiple]
 
     # [START howto_hitl_operator_timeout]
     wait_for_default_option = HITLOperator(
@@ -127,8 +132,8 @@ with DAG(
         body="""
         Input: {{ ti.xcom_pull(task_ids='wait_for_input')["params_input"]["information"] }}
         Option: {{ ti.xcom_pull(task_ids='wait_for_option')["chosen_options"] }}
-        Multiple Options: {{ ti.xcom_pull(task_ids='wait_for_option')["chosen_options"] }}
-        Timeout Option: {{ ti.xcom_pull(task_ids='wait_for_option')["chosen_options"] }}
+        Multiple Options: {{ ti.xcom_pull(task_ids='wait_for_multiple_options')["chosen_options"] }}
+        Timeout Option: {{ ti.xcom_pull(task_ids='wait_for_default_option')["chosen_options"] }}
         """,
         defaults="Reject",
         execution_timeout=datetime.timedelta(minutes=1),
