@@ -20,26 +20,28 @@ import logging
 
 from airflow.exceptions import AirflowOptionalProviderFeatureException
 from airflow.providers.standard.version_compat import AIRFLOW_V_3_1_PLUS
-from airflow.sdk.bases.notifier import BaseNotifier
 
 if not AIRFLOW_V_3_1_PLUS:
     raise AirflowOptionalProviderFeatureException("Human in the loop functionality needs Airflow 3.1+.")
 
-
 from collections.abc import Collection, Mapping, Sequence
 from typing import TYPE_CHECKING, Any
+from urllib.parse import ParseResult, urlencode, urlparse, urlunparse
 
+from airflow.configuration import conf
 from airflow.providers.standard.exceptions import HITLTimeoutError, HITLTriggerEventError
 from airflow.providers.standard.operators.branch import BranchMixIn
 from airflow.providers.standard.triggers.hitl import HITLTrigger, HITLTriggerEventSuccessPayload
 from airflow.providers.standard.utils.skipmixin import SkipMixin
 from airflow.providers.standard.version_compat import BaseOperator
+from airflow.sdk.bases.notifier import BaseNotifier
 from airflow.sdk.definitions.param import ParamsDict
 from airflow.sdk.execution_time.hitl import upsert_hitl_detail
 from airflow.sdk.timezone import utcnow
 
 if TYPE_CHECKING:
     from airflow.sdk.definitions.context import Context
+    from airflow.sdk.types import RuntimeTaskInstanceProtocol
 
 
 class HITLOperator(BaseOperator):
@@ -68,6 +70,7 @@ class HITLOperator(BaseOperator):
         multiple: bool = False,
         params: ParamsDict | dict[str, Any] | None = None,
         notifiers: Sequence[BaseNotifier] | BaseNotifier | None = None,
+        respondents: str | list[str] | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -83,14 +86,32 @@ class HITLOperator(BaseOperator):
         self.notifiers: Sequence[BaseNotifier] = (
             [notifiers] if isinstance(notifiers, BaseNotifier) else notifiers or []
         )
+        self.respondents = [respondents] if isinstance(respondents, str) else respondents
 
         self.validate_options()
+        self.validate_params()
         self.validate_defaults()
 
     def validate_options(self) -> None:
+        """
+        Validate the `options` attribute of the instance.
+
+        Raises:
+            ValueError: If `options` is empty.
+        """
         if not self.options:
             raise ValueError('"options" cannot be empty.')
 
+    def validate_params(self) -> None:
+        """
+        Validate the `params` attribute of the instance.
+
+        Raises:
+            ValueError: If `"_options"` key is present in `params`, which is not allowed.
+        """
+        if "_options" in self.params:
+            raise ValueError('"_options" is not allowed in params')
+
     def validate_defaults(self) -> None:
         """
         Validate whether the given defaults pass the following criteria.
@@ -117,6 +138,7 @@ class HITLOperator(BaseOperator):
             defaults=self.defaults,
             multiple=self.multiple,
             params=self.serialized_params,
+            respondents=self.respondents,
         )
 
         if self.execution_timeout:
@@ -178,6 +200,95 @@ class HITLOperator(BaseOperator):
         ):
             raise ValueError(f"params_input {params_input} does not match params {self.params}")
 
+    def generate_link_to_ui(
+        self,
+        *,
+        task_instance: RuntimeTaskInstanceProtocol,
+        base_url: str | None = None,
+        options: str | list[str] | None = None,
+        params_input: dict[str, Any] | None = None,
+    ) -> str:
+        """
+        Generate a URL link to the "required actions" page for a specific task instance.
+
+        This URL includes query parameters based on allowed options and parameters.
+
+        Args:
+            task_instance: The task instance to generate the link for.
+            base_url: Optional base URL to use. Defaults to ``api.base_url`` from config.
+            options: Optional subset of allowed options to include in the URL.
+            params_input: Optional subset of allowed params to include in the URL.
+
+        Raises:
+            ValueError: If any provided option or parameter is invalid.
+            ValueError: If no base_url can be determined.
+
+        Returns:
+            The full URL pointing to the required actions page with query parameters.
+        """
+        query_param: dict[str, Any] = {}
+        options = [options] if isinstance(options, str) else options
+        if options:
+            if diff := set(options) - set(self.options):
+                raise ValueError(f"options {diff} are not valid options")
+            query_param["_options"] = options
+
+        if params_input:
+            if diff := set(params_input.keys()) - set(self.params.keys()):
+                raise ValueError(f"params {diff} are not valid params")
+            query_param.update(params_input)
+
+        if not (base_url := base_url or conf.get("api", "base_url", fallback=None)):
+            raise ValueError("Not able to retrieve base_url")
+
+        query_param["map_index"] = task_instance.map_index
+
+        parsed_base_url: ParseResult = urlparse(base_url)
+        return urlunparse(
+            (
+                parsed_base_url.scheme,
+                parsed_base_url.netloc,
+                f"/dags/{task_instance.dag_id}/runs/{task_instance.run_id}/tasks/{task_instance.task_id}/required_actions",
+                "",
+                urlencode(query_param) if query_param else "",
+                "",
+            )
+        )
+
+    @staticmethod
+    def generate_link_to_ui_from_context(
+        *,
+        context: Context,
+        base_url: str | None = None,
+        options: list[str] | None = None,
+        params_input: dict[str, Any] | None = None,
+    ) -> str:
+        """
+        Generate a "required actions" page URL from a task context.
+
+        Delegates to ``generate_link_to_ui`` using the task and task_instance extracted from
+        the provided context.
+
+        Args:
+            context: The Airflow task context containing 'task' and 'task_instance'.
+            base_url: Optional base URL to use.
+            options: Optional list of allowed options to include.
+            params_input: Optional dictionary of allowed parameters to include.
+
+        Returns:
+            The full URL pointing to the required actions page with query parameters.
+        """
+        hitl_op = context["task"]
+        if not isinstance(hitl_op, HITLOperator):
+            raise ValueError("This method only supports HITLOperator")
+
+        return hitl_op.generate_link_to_ui(
+            task_instance=context["task_instance"],
+            base_url=base_url,
+            options=options,
+            params_input=params_input,
+        )
+
 
 class ApprovalOperator(HITLOperator, SkipMixin):
     """Human-in-the-loop Operator that has only 'Approval' and 'Reject' options."""
@@ -239,9 +350,57 @@ class HITLBranchOperator(HITLOperator, BranchMixIn):
 
     inherits_from_skipmixin = True
 
+    def __init__(self, *, options_mapping: dict[str, str] | None = None, **kwargs) -> None:
+        """
+        Initialize HITLBranchOperator.
+
+        Args:
+            options_mapping:
+                A dictionary mapping option labels (must match entries in `self.options`)
+                to string values (e.g., task IDs). Defaults to an empty dict if not provided.
+
+        Raises:
+            ValueError:
+                - If `options_mapping` contains keys not present in `self.options`.
+                - If any value in `options_mapping` is not a string.
+        """
+        super().__init__(**kwargs)
+        self.options_mapping = options_mapping or {}
+        self.validate_options_mapping()
+
+    def validate_options_mapping(self) -> None:
+        """
+        Validate that `options_mapping` keys match `self.options` and all values are strings.
+
+        Raises:
+            ValueError: If any key is not in `self.options` or any value is not a string.
+        """
+        if not self.options_mapping:
+            return
+
+        # Validate that the choice options are keys in the mapping are the same
+        invalid_keys = set(self.options_mapping.keys()) - set(self.options)
+        if invalid_keys:
+            raise ValueError(
+                f"`options_mapping` contains keys that are not in `options`: {sorted(invalid_keys)}"
+            )
+
+        # validate that all values are strings
+        invalid_entries = {
+            k: (v, type(v).__name__) for k, v in self.options_mapping.items() if not isinstance(v, str)
+        }
+        if invalid_entries:
+            raise ValueError(
+                f"`options_mapping` values must be strings (task_ids).\nInvalid entries: {invalid_entries}"
+            )
+
     def execute_complete(self, context: Context, event: dict[str, Any]) -> Any:
+        """Execute the operator and branch based on chosen options."""
         ret = super().execute_complete(context=context, event=event)
         chosen_options = ret["chosen_options"]
+
+        # Map options to task IDs using the mapping, fallback to original option
+        chosen_options = [self.options_mapping.get(option, option) for option in chosen_options]
         return self.do_branch(context=context, branches_to_execute=chosen_options)
 
 
